/*
 *	Contains all the Type Definitions and functions declarations
 *	for the CUnit test database maintenance.
 *
 *	Created By     : Anil Kumar on ...(in month of Aug 2001)
 *	Last Modified  : 09/Aug/2001
 *	Comment        : Added Preprocessor conditionals for the file.
 *	EMail          : aksaharan@yahoo.com
 *
 *	Last Modified  : 24/aug/2001 by Anil Kumar
 *	Comment        : Made the linked list from SLL to DLL(doubly linked list).
 *	EMail          : aksaharan@yahoo.com
 */

#ifndef _TestDB_H_
#define _TestDB_H_

#include "CUnit.h"
#include "Errno.h"
/*
 *	Type definition for Initialization/Cleaup/TestFunction for TestCase/TestGroup
 */
typedef int (*InitializeFunc)(void);
typedef int (*CleanupFunc)(void);
typedef int (*InitializeTest)(void);
typedef int (*CleanupTest)(void);
typedef void (*TestFunc)(void);

typedef struct _TestCase
{
	char*			pName;
	TestFunc		pTestFunc;

	struct _TestCase*		pNext;
	struct _TestCase*		pPrev;

}TestCase;
typedef TestCase *PTestCase;

typedef struct _TestGroup
{
	char*			pName;
	PTestCase		pTestCase;
	InitializeFunc	pInitializeFunc;
	CleanupFunc		pCleanupFunc;
	InitializeTest	pInitializeTest;
	CleanupTest		pCleanupTest;

	unsigned int	uiNumberOfTests;
	struct _TestGroup*		pNext;
	struct _TestGroup*		pPrev;

}TestGroup;
typedef TestGroup *PTestGroup;

typedef struct _TestResult
{
	unsigned int	uiLineNumber;
	char*			strFileName;
	char*			strCondition;	
	PTestCase		pTestCase;
	PTestGroup		pTestGroup;

	struct _TestResult*		pNext;
	struct _TestResult*		pPrev;

}TestResult;
typedef TestResult *PTestResult;

typedef struct _TestRegistry
{
	unsigned int	uiNumberOfGroups;
	unsigned int	uiNumberOfTests;
	unsigned int	uiNumberOfFailures;

	PTestGroup		pGroup;
	PTestResult		pResult;
}TestRegistry;
typedef TestRegistry *PTestRegistry;

extern PTestRegistry g_pTestRegistry;
extern int error_number;

extern int initialize_registry(void) FRAMEWORK;
extern void cleanup_registry(void) FRAMEWORK;

extern PTestRegistry get_registry(void) FRAMEWORK;
extern int set_registry(PTestRegistry pTestRegistry) FRAMEWORK;


extern PTestGroup add_test_group(char* strName, InitializeFunc pInit, CleanupFunc pClean, InitializeTest pTInit, CleanupTest pTClean) FRAMEWORK;
extern PTestCase add_test_case(PTestGroup pGroup, char* strName, TestFunc pTest) FRAMEWORK;

/*
 * This function is for internal use and is used by the 
 * Asssert Implementation function to store the error description
 * and the codes.
 */
extern void add_failure(char szMessage[], char szCondition[], char szFileName[], unsigned int uiLineNumber, PTestGroup pGroup, PTestCase pTest) FRAMEWORK;

extern const char* get_error(void) FRAMEWORK;

#endif  /*  _TestDB_H_  */
