/*
 * TestSuite.c
 *
 *   counter_XXX, rcounter_XXX
 */

#include "counter.h"
#include "segments.h"

#include "TestDB.h"

extern void initialize_suite();
extern void terminate_suite();

/*-----------------------------*/

struct counter* c;
 
static int initializeGroup() {
  c = counter_initialize();
  return c == 0;
}

static int terminateGroup() {
  counter_terminate(c);
  return 0;
}

static int initializeTest() {
  return 0;
}

static int terminateTest() {
  return 0;
}

static void test_clear() {
  counter_clear(c);
  ASSERT_MSG("Clear should zero the value",0 == counter_get_value(c) );
}

static void test_increment() {
  counter_clear(c);
  counter_increment(c);
  ASSERT_INT16_EQUAL(1,counter_get_value(c) );
}

static void test_decrement() {
  counter_clear(c);
  counter_decrement(c);
  ASSERT_INT16_EQUAL_MSG("Counter should be -1 when decremented from 0",-1,counter_get_value(c) );
}

/*-----------------------------*/

struct rcounter* rc;
 
static int rinitializeGroup() {
  rc = rcounter_initialize();
  return rc == 0;
}

static int rterminateGroup() {
  rcounter_terminate(rc);
  return 0;
}

static int rinitializeTest() {
  return 0;
}

static int rterminateTest() {
  return 0;
}


static void test_rclear() {
  rcounter_clear(rc);
  ASSERT( 0 == rcounter_get_value(rc) );
}

static void test_rincrement() {
  rcounter_clear(rc);
  rcounter_increment(rc);
  ASSERT_INT16_EQUAL(-1,rcounter_get_value(rc) );
}

static void test_rdecrement() {
  rcounter_clear(rc);
  rcounter_decrement(rc);
  ASSERT_MSG("Reverse counter should be 1 when decremented from 0",1 == rcounter_get_value(rc) );
}

/*-----------------------------*/

static int initializeCUnitGroup() {
  return 0;
}

static int terminateCUnitGroup() {
  return 0;
}

static int initializeCUnitTest() {
  return 0;
}

static int terminateCUnitTest() {
  return 0;
}

static void test_Int8() {
  Int8 i=0;
  Int8 j=-5;
  i--;

  ASSERT_INT8_EQUAL(i,j)
}
static void test_UInt8() {
  UInt8 i=0;
  UInt8 j=5;
  i--;

  ASSERT_UINT8_EQUAL(i,j)
}
static void test_Int16() {
  Int16 i=0;
  Int16 j=-5;
  i--;

  ASSERT_INT16_EQUAL(i,j)
}
static void test_UInt16() {
  UInt16 i=0;
  UInt16 j=5;
  i--;

  ASSERT_UINT16_EQUAL(i,j)
}
static void test_Int32() {
  Int32 i=0;
  Int32 j=-5;
  i--;

  ASSERT_INT32_EQUAL(i,j)
}
static void test_UInt32() {
  UInt32 i=0;
  UInt32 j=5;
  i--;

  ASSERT_UINT32_EQUAL(i,j)
}
static void test_String() {
  Char *i = "This";
  Char *j = "That";

  ASSERT_STR_EQUAL(i,j)
}

/*-----------------------------*/
 
void initialize_suite() {
  TestGroup* group;
  group = add_test_group("normal", initializeGroup, terminateGroup, initializeTest, terminateTest);
  add_test_case(group, "clear",     test_clear);
  add_test_case(group, "increment", test_increment);
  add_test_case(group, "decrement", test_decrement);

  group = add_test_group("reverse", rinitializeGroup, rterminateGroup, rinitializeTest, rterminateTest);
  add_test_case(group, "rclear",     test_rclear);
  add_test_case(group, "rincrement", test_rincrement);
  add_test_case(group, "rdecrement", test_rdecrement);

  group = add_test_group("CUnit", initializeCUnitGroup, terminateCUnitGroup, initializeCUnitTest, terminateCUnitTest);
  add_test_case(group, "Int8",   test_Int8);
  add_test_case(group, "UInt8",  test_UInt8);
  add_test_case(group, "Int16",  test_Int16);
  add_test_case(group, "UInt16", test_UInt16);
  add_test_case(group, "Int32",  test_Int32);
  add_test_case(group, "UInt32", test_UInt32);
  add_test_case(group, "String", test_String);
}

void terminate_suite() {
}
